def main(request, response):
    origin = request.GET.first(b"origin");
    value = request.GET.first(b"value");
    # This is used to solve the race condition we have for postMessages
    shouldSucceed = request.GET.first(b"loadShouldSucceed", b"false");
    return ([(b"Content-Type", b"text/html")],
            b"""<!DOCTYPE html>
<title>XFO.</title>
<body>
<script>
  var gotMessage = false;
  window.addEventListener("message", e => {
    gotMessage = true;
    window.parent.postMessage(e.data, "*");
  });

  var i = document.createElement("iframe");
  i.src = "%s/x-frame-options/support/xfo.py?value=%s";
  i.onload = _ => {
    // Why 100ms timeout? Because that seems to be enough to stop the
    // load event from racing with the onmessage event, and it's at least
    // as long as the two renderAnimationFrame calls that used to be here.
    setTimeout(_ => {
      // The race condition problem we have is it is possible
      // that the sub iframe is loaded before the postMessage is
      // dispatched, as a result, the "Failed" message is sent
      // out. So the way we fixed is we simply let the timeout
      // to happen if we expect the "Loaded" postMessage to be
      // sent
      if (!gotMessage && %s != true) {
        window.parent.postMessage("Failed", "*");
      }
    }, 100);
  };
  document.body.appendChild(i);
</script>
            """ % (origin, value, shouldSucceed))




import pytest

from webdriver.error import NoSuchAlertException
from webdriver.transport import Response

from tests.support.asserts import assert_error, assert_success
from tests.support.sync import Poll
from . import execute_script


def test_null_parameter_value(session, http):
    path = "/session/{session_id}/execute/sync".format(**vars(session))
    with http.post(path, None) as response:
        assert_error(Response.from_http(response), "invalid argument")


def test_no_top_browsing_context(session, closed_window):
    response = execute_script(session, "return 1;")
    assert_error(response, "no such window")


def test_no_browsing_context(session, closed_frame):
    response = execute_script(session, "return 1;")
    assert_error(response, "no such window")


@pytest.mark.parametrize("expression, expected", [
    ("null", None),
    ("undefined", None),
    ("true", True),
    ("false", False),
    ("23", 23),
    ("'foo'", "foo"),
    (
        # Compute value in the runtime to reduce the potential for
        # interference from encoding literal bytes or escape sequences in
        # Python and HTTP.
        "String.fromCharCode(0)",
        "\x00"
    )
])
def test_primitive_serialization(session, expression, expected):
    response = execute_script(session, "return {};".format(expression))
    value = assert_success(response)
    assert value == expected


def test_opening_new_window_keeps_current_window_handle(session, inline):
    original_handle = session.window_handle
    original_handles = session.handles

    url = inline("""<a href="javascript:window.open();">open window</a>""")
    session.url = url
    session.find.css("a", all=False).click()
    wait = Poll(
        session,
        timeout=5,
        message="No new window has been opened")
    new_handles = wait.until(lambda s: set(s.handles) - set(original_handles))

    assert len(new_handles) == 1
    assert session.window_handle == original_handle
    assert session.url == url


def test_ending_comment(session):
    response = execute_script(session, "return 1; // foo")
    assert_success(response, 1)


def test_override_listeners(session, inline):
    session.url = inline("""
<script>
called = [];
window.addEventListener = () => {called.push("Internal addEventListener")}
window.removeEventListener = () => {called.push("Internal removeEventListener")}
</script>
})""")
    response = execute_script(session, "return !window.onunload")
    assert_success(response, True)
    response = execute_script(session, "return called")
    assert_success(response, [])


@pytest.mark.parametrize("dialog_type", ["alert", "confirm", "prompt"])
def test_abort_by_user_prompt(session, dialog_type):
    response = execute_script(
        session, "window.{}('Hello'); return 1;".format(dialog_type))
    assert_success(response, None)

    session.alert.accept()


@pytest.mark.parametrize("dialog_type", ["alert", "confirm", "prompt"])
def test_abort_by_user_prompt_twice(session, dialog_type):
    response = execute_script(
        session, "window.{0}('Hello'); window.{0}('Bye'); return 1;".format(dialog_type))
    assert_success(response, None)

    session.alert.accept()

    # The first alert has been accepted by the user prompt handler, the second
    # alert will still be opened because the current step isn't aborted.
    wait = Poll(
        session,
        timeout=5,
        message="Second alert has not been opened",
        ignored_exceptions=NoSuchAlertException
    )
    text = wait.until(lambda s: s.alert.text)

    assert text == "Bye"

    session.alert.accept()



import pytest
from tests.support import platform_name
from webdriver.bidi.modules.script import ContextTarget

pytestmark = pytest.mark.asyncio


@pytest.mark.skipif(
    platform_name is None, reason="Unsupported platform {}".format(platform_name)
)
@pytest.mark.parametrize("match_type", ["alwaysMatch", "firstMatch"])
async def test_platform_name(new_session, match_capabilities, match_type):
    capabilities = match_capabilities(match_type, "platformName", platform_name)

    bidi_session = await new_session(capabilities=capabilities)

    assert bidi_session.capabilities["platformName"] == platform_name


@pytest.mark.parametrize("match_type", ["alwaysMatch", "firstMatch"])
async def test_proxy(
    new_session, match_capabilities, server_config, inline, match_type
):
    domain = server_config["domains"][""][""]
    port = server_config["ports"]["http"][0]
    proxy_url = f"{domain}:{port}"
    proxy_capability = {"proxyType": "manual", "httpProxy": proxy_url}
    capabilities = match_capabilities(match_type, "proxy", proxy_capability)

    bidi_session = await new_session(capabilities=capabilities)

    assert bidi_session.capabilities["proxy"] == proxy_capability

    page_content = "proxy"
    page_url = inline(f"<div>{page_content}</div>")
    test_url = page_url.replace(proxy_url, "example.com")

    contexts = await bidi_session.browsing_context.get_tree()

    await bidi_session.browsing_context.navigate(
        context=contexts[0]["context"], url=test_url, wait="complete"
    )

    # Check that content is expected
    response = await bidi_session.script.evaluate(
        expression="""document.querySelector('div').textContent""",
        target=ContextTarget(contexts[0]["context"]),
        await_promise=False,
    )

    assert response == {"type": "string", "value": page_content}


@pytest.mark.parametrize("match_type", ["alwaysMatch", "firstMatch"])
async def test_websocket_url(new_session, match_capabilities, match_type):
    capabilities = match_capabilities(match_type, "webSocketUrl", True)

    bidi_session = await new_session(capabilities=capabilities)

    assert bidi_session.capabilities.get("webSocketUrl") is None



# This file exists to allow `python wpt <command>` to work on Windows:
# https://github.com/web-platform-tests/wpt/pull/6907 and
# https://github.com/web-platform-tests/wpt/issues/23095
import os
abspath = os.path.abspath(__file__)
os.chdir(os.path.dirname(abspath))
exec(compile(open("wpt", "r").read(), "wpt", 'exec'))



def main(request, response):
    response.status = 302
    if b"location" in request.GET:
        response.headers.set(b"Location", request.GET[b"location"])
    else:
        response.headers.set(b"Location", b"post_message_to_frame_owner.html")



import pytest

from webdriver.error import InvalidArgumentException, NoSuchWindowException, StaleElementReferenceException

from tests.classic.perform_actions.support.mouse import (
    get_inview_center,
    get_viewport_rect,
)
from tests.classic.perform_actions.support.refine import get_events
from tests.support.asserts import assert_move_to_coordinates
from tests.support.helpers import filter_dict
from tests.support.sync import Poll

from . import assert_pointer_events, record_pointer_events


def test_null_response_value(session, mouse_chain):
    value = mouse_chain.click().perform()
    assert value is None


def test_no_top_browsing_context(session, closed_window, mouse_chain):
    with pytest.raises(NoSuchWindowException):
        mouse_chain.click().perform()


def test_no_browsing_context(session, closed_frame, mouse_chain):
    with pytest.raises(NoSuchWindowException):
        mouse_chain.click().perform()


@pytest.mark.parametrize("as_frame", [False, True], ids=["top_context", "child_context"])
def test_stale_element_reference(session, stale_element, mouse_chain, as_frame):
    element = stale_element("input#text", as_frame=as_frame)

    with pytest.raises(StaleElementReferenceException):
        mouse_chain.click(element=element).perform()


def test_click_at_coordinates(session, test_actions_page, mouse_chain):
    div_point = {
        "x": 82,
        "y": 187,
    }
    mouse_chain \
        .pointer_move(div_point["x"], div_point["y"], duration=1000) \
        .click() \
        .perform()
    events = get_events(session)
    assert len(events) == 4
    assert_move_to_coordinates(div_point, "outer", events)
    for e in events:
        if e["type"] != "mousedown":
            assert e["buttons"] == 0
        assert e["button"] == 0
    expected = [
        {"type": "mousedown", "buttons": 1},
        {"type": "mouseup", "buttons": 0},
        {"type": "click", "buttons": 0},
    ]
    filtered_events = [filter_dict(e, expected[0]) for e in events]
    assert expected == filtered_events[1:]


def test_context_menu_at_coordinates(session, test_actions_page, mouse_chain):
    div_point = {
        "x": 82,
        "y": 187,
    }
    mouse_chain \
        .pointer_move(div_point["x"], div_point["y"]) \
        .pointer_down(button=2) \
        .pointer_up(button=2) \
        .perform()

    events = get_events(session)
    assert len(events) == 4

    expected = [
        {"type": "mousedown", "button": 2, "buttons": 2},
        {"type": "contextmenu", "button": 2, "buttons": 2},
    ]
    # Some browsers in some platforms may dispatch `contextmenu` event as a
    # a default action of `mouseup`.  In the case, `.buttons` of the event
    # should be 0.
    anotherExpected = [
        {"type": "mousedown", "button": 2, "buttons": 2},
        {"type": "contextmenu", "button": 2, "buttons": 0},
    ]
    filtered_events = [filter_dict(e, expected[0]) for e in events]
    mousedown_contextmenu_events = [
        x for x in filtered_events
        if x["type"] in ["mousedown", "contextmenu"]
    ]
    assert mousedown_contextmenu_events in [expected, anotherExpected]


def test_middle_click(session, test_actions_page, mouse_chain):
    div_point = {
        "x": 82,
        "y": 187,
    }
    mouse_chain \
        .pointer_move(div_point["x"], div_point["y"]) \
        .pointer_down(button=1) \
        .pointer_up(button=1) \
        .perform()

    events = get_events(session)
    assert len(events) == 3

    expected = [
        {"type": "mousedown", "button": 1, "buttons": 4},
        {"type": "mouseup", "button": 1, "buttons": 0},
    ]
    filtered_events = [filter_dict(e, expected[0]) for e in events]
    mousedown_mouseup_events = [
        x for x in filtered_events
        if x["type"] in ["mousedown", "mouseup"]
    ]
    assert expected == mousedown_mouseup_events


def test_click_element_center(session, test_actions_page, mouse_chain):
    outer = session.find.css("#outer", all=False)
    center = get_inview_center(outer.rect, get_viewport_rect(session))
    mouse_chain.click(element=outer).perform()
    events = get_events(session)
    assert len(events) == 4
    event_types = [e["type"] for e in events]
    assert ["mousemove", "mousedown", "mouseup", "click"] == event_types
    for e in events:
        if e["type"] != "mousemove":
            assert e["pageX"] == pytest.approx(center["x"], abs=1.0)
            assert e["pageY"] == pytest.approx(center["y"], abs=1.0)
            assert e["target"] == "outer"


@pytest.mark.parametrize("mode", ["open", "closed"])
@pytest.mark.parametrize("nested", [False, True], ids=["outer", "inner"])
def test_click_element_in_shadow_tree(
    session, get_test_page, mouse_chain, mode, nested
):
    session.url = get_test_page(
        shadow_doc="""
        <div id="pointer-target"
             style="width: 10px; height: 10px; background-color:blue;">
        </div>""",
        shadow_root_mode=mode,
        nested_shadow_dom=nested,
    )

    shadow_root = session.find.css("custom-element", all=False).shadow_root
    if nested:
        shadow_root = shadow_root.find_element("css selector", "inner-custom-element").shadow_root

    target = shadow_root.find_element("css selector", "#pointer-target")
    record_pointer_events(session, target)

    mouse_chain.click(element=target).perform()
    assert_pointer_events(
        session,
        expected_events=["pointerdown", "pointerup"],
        target="pointer-target",
        pointer_type="mouse",
    )


def test_click_navigation(session, url, inline):
    destination = url("/webdriver/tests/support/html/test_actions.html")
    start = inline("<a href=\"{}\" id=\"link\">destination</a>".format(destination))

    def click(link):
        mouse_chain = session.actions.sequence(
            "pointer", "pointer_id", {"pointerType": "mouse"})
        mouse_chain.click(element=link).perform()

    session.url = start
    error_message = "Did not navigate to %s" % destination

    click(session.find.css("#link", all=False))
    Poll(session, message=error_message).until(lambda s: s.url == destination)
    # repeat steps to check behaviour after document unload
    session.url = start
    click(session.find.css("#link", all=False))
    Poll(session, message=error_message).until(lambda s: s.url == destination)


@pytest.mark.parametrize("x, y, event_count", [
    (0, 0, 0),
    (1, 0, 1),
    (0, 1, 1),
], ids=["default value", "x", "y"])
def test_move_to_position_in_viewport(
    session, test_actions_page, mouse_chain, x, y, event_count
):
    mouse_chain.pointer_move(x, y).perform()
    events = get_events(session)
    assert len(events) == event_count

    # Move again to check that no further mouse move event is emitted.
    mouse_chain.pointer_move(x, y).perform()
    events = get_events(session)
    assert len(events) == event_count


@pytest.mark.parametrize("drag_duration", [0, 300, 800])
@pytest.mark.parametrize("dx, dy", [
    (20, 0), (0, 15), (10, 15), (-20, 0), (10, -15), (-10, -15)
])
def test_drag_and_drop(session,
                       test_actions_page,
                       mouse_chain,
                       dx,
                       dy,
                       drag_duration):
    drag_target = session.find.css("#dragTarget", all=False)
    initial_rect = drag_target.rect
    initial_center = get_inview_center(initial_rect, get_viewport_rect(session))
    # Conclude chain with extra move to allow time for last queued
    # coordinate-update of drag_target and to test that drag_target is "dropped".
    mouse_chain \
        .pointer_move(0, 0, origin=drag_target) \
        .pointer_down() \
        .pointer_move(dx, dy, duration=drag_duration, origin="pointer") \
        .pointer_up() \
        .pointer_move(80, 50, duration=100, origin="pointer") \
        .perform()
    # mouseup that ends the drag is at the expected destination
    e = get_events(session)[1]
    assert e["type"] == "mouseup"
    assert e["pageX"] == pytest.approx(initial_center["x"] + dx, abs=1.0)
    assert e["pageY"] == pytest.approx(initial_center["y"] + dy, abs=1.0)
    # check resulting location of the dragged element
    final_rect = drag_target.rect
    assert initial_rect["x"] + dx == final_rect["x"]
    assert initial_rect["y"] + dy == final_rect["y"]


@pytest.mark.parametrize("drag_duration", [0, 300, 800])
def test_drag_and_drop_with_draggable_element(session_new_window,
                       test_actions_page,
                       mouse_chain,
                       drag_duration):
    new_session = session_new_window
    drag_target = new_session.find.css("#draggable", all=False)
    drop_target = new_session.find.css("#droppable", all=False)
    # Conclude chain with extra move to allow time for last queued
    # coordinate-update of drag_target and to test that drag_target is "dropped".
    mouse_chain \
        .pointer_move(0, 0, origin=drag_target) \
        .pointer_down() \
        .pointer_move(50,
                      25,
                      duration=drag_duration,
                      origin=drop_target) \
        .pointer_up() \
        .pointer_move(80, 50, duration=100, origin="pointer") \
        .perform()
    # mouseup that ends the drag is at the expected destination
    e = get_events(new_session)
    assert len(e) >= 5
    assert e[1]["type"] == "dragstart", "Events captured were {}".format(e)
    assert e[2]["type"] == "dragover", "Events captured were {}".format(e)
    drag_events_captured = [
        ev["type"] for ev in e if ev["type"].startswith("drag") or ev["type"].startswith("drop")
    ]
    assert "dragend" in drag_events_captured
    assert "dragenter" in drag_events_captured
    assert "dragleave" in drag_events_captured
    assert "drop" in drag_events_captured


@pytest.mark.parametrize("missing", ["x", "y"])
def test_missing_coordinates(session, test_actions_page, mouse_chain, missing):
    outer = session.find.css("#outer", all=False)
    actions = mouse_chain.pointer_move(x=0, y=0, origin=outer)
    del actions._actions[-1][missing]
    with pytest.raises(InvalidArgumentException):
        actions.perform()


def test_invalid_element_origin(session, test_actions_page, mouse_chain):
    outer = session.find.css("#outer", all=False)
    actions = mouse_chain.pointer_move(
        x=0, y=0, origin={"type": "element", "element": {"sharedId": outer.id}}
    )
    with pytest.raises(InvalidArgumentException):
        actions.perform()



import pytest
import pytest_asyncio

from .helpers import (
    Browser,
    Geckodriver,
    create_custom_profile,
    get_pref,
    get_profile_folder,
    read_user_preferences,
    set_pref,
)


@pytest.fixture(scope="module")
def browser(configuration, firefox_options):
    """Start a Firefox instance without using geckodriver.

    geckodriver will automatically use the --remote-allow-hosts and
    --remote.allow.origins command line arguments.

    Starting Firefox without geckodriver allows to set those command line arguments
    as needed. The fixture method returns the browser instance that should be used
    to connect to a RemoteAgent supported protocol (CDP, WebDriver BiDi).
    """
    current_browser = None

    def _browser(
        use_bidi=False,
        use_cdp=False,
        extra_args=None,
        extra_prefs=None,
        clone_profile=True,
    ):
        nonlocal current_browser

        # If the requested preferences and arguments match the ones for the
        # already started firefox, we can reuse the current firefox instance,
        # return the instance immediately.
        if current_browser:
            if (
                current_browser.use_bidi == use_bidi
                and current_browser.use_cdp == use_cdp
                and current_browser.extra_args == extra_args
                and current_browser.extra_prefs == extra_prefs
                and current_browser.is_running
            ):
                return current_browser

            # Otherwise, if firefox is already started, terminate it because we need
            # to create a new instance for the provided preferences.
            current_browser.quit()

        binary = configuration["browser"]["binary"]
        env = configuration["browser"]["env"]

        profile_path = get_profile_folder(firefox_options)
        default_prefs = read_user_preferences(profile_path)
        profile = create_custom_profile(
            profile_path, default_prefs, clone=clone_profile
        )

        current_browser = Browser(
            binary,
            profile,
            use_bidi=use_bidi,
            use_cdp=use_cdp,
            extra_args=extra_args,
            extra_prefs=extra_prefs,
            env=env,
        )
        current_browser.start()
        return current_browser

    yield _browser

    # Stop firefox at the end of the test module.
    if current_browser is not None:
        current_browser.quit()
        current_browser = None


@pytest.fixture(name="create_custom_profile")
def fixture_create_custom_profile(default_preferences, profile_folder):
    profile = None

    def _create_custom_profile(clone=True):
        profile = create_custom_profile(
            profile_folder, default_preferences, clone=clone
        )

        return profile

    yield _create_custom_profile

    # if profile is not None:
    if profile:
        profile.cleanup()


@pytest.fixture
def default_preferences(profile_folder):
    return read_user_preferences(profile_folder)


@pytest.fixture(scope="session")
def firefox_options(configuration):
    return configuration["capabilities"]["moz:firefoxOptions"]


@pytest_asyncio.fixture
async def geckodriver(configuration):
    """Start a geckodriver instance directly."""
    driver = None

    def _geckodriver(config=None, hostname=None, extra_args=None):
        nonlocal driver

        if config is None:
            config = configuration

        driver = Geckodriver(config, hostname, extra_args)
        driver.start()

        return driver

    yield _geckodriver

    if driver is not None:
        await driver.stop()


@pytest.fixture
def profile_folder(firefox_options):
    return get_profile_folder(firefox_options)


@pytest.fixture
def use_pref(session):
    """Set a specific pref value."""
    reset_values = {}

    def _use_pref(pref, value):
        if pref not in reset_values:
            reset_values[pref] = get_pref(session, pref)

        set_pref(session, pref, value)

    yield _use_pref

    for pref, reset_value in reset_values.items():
        set_pref(session, pref, reset_value)



import asyncio
import pytest_asyncio

from webdriver.bidi.error import NoSuchInterceptException

from . import PAGE_EMPTY_TEXT


@pytest_asyncio.fixture
async def add_intercept(bidi_session):
    """Add a network intercept for the provided phases and url patterns, and
    ensure the intercept is removed at the end of the test."""

    intercepts = []

    async def add_intercept(phases, url_patterns, contexts = None):
        nonlocal intercepts
        intercept = await bidi_session.network.add_intercept(
            phases=phases,
            url_patterns=url_patterns,
            contexts=contexts,
        )
        intercepts.append(intercept)

        return intercept

    yield add_intercept

    # Remove all added intercepts at the end of the test
    for intercept in intercepts:
        try:
            await bidi_session.network.remove_intercept(intercept=intercept)
        except NoSuchInterceptException:
            # Ignore exceptions in case a specific intercept was already removed
            # during the test.
            pass


@pytest_asyncio.fixture
async def setup_blocked_request(
    bidi_session,
    setup_network_test,
    url,
    add_intercept,
    fetch,
    wait_for_event,
    wait_for_future_safe,
    top_context,
):
    """Creates an intercept for the provided phase, sends a fetch request that
    should be blocked by this intercept and resolves when the corresponding
    event is received. Pass navigate=True in order to navigate instead of doing
    a fetch request.

    For the "authRequired" phase, the request will be sent to the authentication
    http handler. The optional arguments username, password and realm can be used
    to configure the handler.

    Returns the `request` id of the intercepted request.
    """

    async def setup_blocked_request(
        phase,
        context=top_context,
        username="user",
        password="password",
        realm="test",
        navigate=False,
        **kwargs,
    ):
        await setup_network_test(events=[f"network.{phase}"])

        if phase == "authRequired":
            blocked_url = url(
                "/webdriver/tests/support/http_handlers/authentication.py?"
                f"username={username}&password={password}&realm={realm}"
            )
            if navigate:
                # By default the authentication handler returns a text/plain
                # content-type. Switch to text/html for a regular navigation.
                blocked_url = f"{blocked_url}&contenttype=text/html"
        else:
            blocked_url = url(PAGE_EMPTY_TEXT)

        await add_intercept(
            phases=[phase],
            url_patterns=[
                {
                    "type": "string",
                    "pattern": blocked_url,
                }
            ],
        )

        network_event = wait_for_event(f"network.{phase}")
        if navigate:
            asyncio.ensure_future(
                bidi_session.browsing_context.navigate(
                    context=context["context"], url=blocked_url, wait="complete"
                )
            )
        else:
            asyncio.ensure_future(fetch(blocked_url, context=context, **kwargs))

        event = await wait_for_future_safe(network_event)
        request = event["request"]["request"]

        return request

    return setup_blocked_request



# META: timeout=long

import pytest
from webdriver import error

from tests.support.asserts import assert_dialog_handled, assert_error, assert_success


def element_click(session, element):
    return session.transport.send(
        "POST", "session/{session_id}/element/{element_id}/click".format(
            session_id=session.session_id,
            element_id=element.id))


@pytest.fixture
def check_beforeunload_implicitly_accepted(session, url):
    def check_beforeunload_implicitly_accepted():
        page_beforeunload = url(
            "/webdriver/tests/support/html/beforeunload.html")
        page_target = url("/webdriver/tests/support/html/default.html")

        session.url = page_beforeunload
        input = session.find.css("input", all=False)
        input.send_keys("bar")

        link = session.find.css("a", all=False)
        response = element_click(session, link)
        assert_success(response)

        assert session.url == page_target

        # navigation auto-dismissed beforeunload prompt
        with pytest.raises(error.NoSuchAlertException):
            session.alert.text

    return check_beforeunload_implicitly_accepted


@pytest.fixture
def check_user_prompt_closed_without_exception(session, create_dialog, inline):
    def check_user_prompt_closed_without_exception(dialog_type, retval):
        session.url = inline("<input type=text>")
        element = session.find.css("input", all=False)

        create_dialog(dialog_type, text=dialog_type)

        response = element_click(session, element)
        assert_success(response)

        assert_dialog_handled(session, expected_text=dialog_type, expected_retval=retval)

        assert session.active_element == element

    return check_user_prompt_closed_without_exception


@pytest.fixture
def check_user_prompt_closed_with_exception(session, create_dialog, inline):
    def check_user_prompt_closed_with_exception(dialog_type, retval):
        session.url = inline("<input type=text>")
        element = session.find.css("input", all=False)

        create_dialog(dialog_type, text=dialog_type)

        response = element_click(session, element)
        assert_error(response, "unexpected alert open")

        assert_dialog_handled(session, expected_text=dialog_type, expected_retval=retval)

        assert session.active_element != element

    return check_user_prompt_closed_with_exception


@pytest.fixture
def check_user_prompt_not_closed_but_exception(session, create_dialog, inline):
    def check_user_prompt_not_closed_but_exception(dialog_type):
        session.url = inline("<input type=text>")
        element = session.find.css("input", all=False)

        create_dialog(dialog_type, text=dialog_type)

        response = element_click(session, element)
        assert_error(response, "unexpected alert open")

        assert session.alert.text == dialog_type
        session.alert.dismiss()

        assert session.active_element != element

    return check_user_prompt_not_closed_but_exception


@pytest.mark.capabilities({"unhandledPromptBehavior": "accept"})
@pytest.mark.parametrize("dialog_type, retval", [
    ("alert", None),
    ("beforeunload", None),
    ("confirm", True),
    ("prompt", ""),
])
def test_accept(
    check_beforeunload_implicitly_accepted,
    check_user_prompt_closed_without_exception,
    dialog_type,
    retval
):
    if dialog_type == "beforeunload":
        check_beforeunload_implicitly_accepted()
    else:
        check_user_prompt_closed_without_exception(dialog_type, retval)


@pytest.mark.capabilities({"unhandledPromptBehavior": "accept and notify"})
@pytest.mark.parametrize("dialog_type, retval", [
    ("alert", None),
    ("beforeunload", None),
    ("confirm", True),
    ("prompt", ""),
])
def test_accept_and_notify(
    check_beforeunload_implicitly_accepted,
    check_user_prompt_closed_with_exception,
    dialog_type,
    retval
):
    if dialog_type == "beforeunload":
        check_beforeunload_implicitly_accepted()
    else:
        check_user_prompt_closed_with_exception(dialog_type, retval)


@pytest.mark.capabilities({"unhandledPromptBehavior": "dismiss"})
@pytest.mark.parametrize("dialog_type, retval", [
    ("alert", None),
    ("beforeunload", None),
    ("confirm", False),
    ("prompt", None),
])
def test_dismiss(
    check_beforeunload_implicitly_accepted,
    check_user_prompt_closed_without_exception,
    dialog_type,
    retval,
):
    if dialog_type == "beforeunload":
        check_beforeunload_implicitly_accepted()
    else:
        check_user_prompt_closed_without_exception(dialog_type, retval)


@pytest.mark.capabilities({"unhandledPromptBehavior": "dismiss and notify"})
@pytest.mark.parametrize("dialog_type, retval", [
    ("alert", None),
    ("beforeunload", None),
    ("confirm", False),
    ("prompt", None),
])
def test_dismiss_and_notify(
    check_beforeunload_implicitly_accepted,
    check_user_prompt_closed_with_exception, dialog_type,
    retval
):
    if dialog_type == "beforeunload":
        check_beforeunload_implicitly_accepted()
    else:
        check_user_prompt_closed_with_exception(dialog_type, retval)


@pytest.mark.capabilities({"unhandledPromptBehavior": "ignore"})
@pytest.mark.parametrize("dialog_type", ["alert", "beforeunload", "confirm", "prompt"])
def test_ignore(
    check_beforeunload_implicitly_accepted,
    check_user_prompt_not_closed_but_exception,
    dialog_type
):
    if dialog_type == "beforeunload":
        check_beforeunload_implicitly_accepted()
    else:
        check_user_prompt_not_closed_but_exception(dialog_type)


@pytest.mark.parametrize("dialog_type, retval", [
    ("alert", None),
    ("beforeunload", None),
    ("confirm", False),
    ("prompt", None),
])
def test_default(
    check_beforeunload_implicitly_accepted,
    check_user_prompt_closed_with_exception,
    dialog_type,
    retval
):
    if dialog_type == "beforeunload":
        check_beforeunload_implicitly_accepted()
    else:
        check_user_prompt_closed_with_exception(dialog_type, retval)



"""Test reporting origin server used for two reasons:

  1. It is a workaround for lack of preflight support in the test server.
  2. Stashes requests so they can be inspected by tests.
"""

from wptserve.stash import Stash
import json

REQUESTS = "9250f93f-2c05-4aae-83b9-2817b0e18b4d"


headers = [
    b"attribution-reporting-eligible",
    b"attribution-reporting-support",
    b"referer",
]


def store_request(request) -> None:
    obj = {
        "method": request.method,
        "url": request.url,
    }
    for header in headers:
        value = request.headers.get(header)
        if value is not None:
            obj[str(header, "utf-8")] = str(value, "utf-8")
    with request.server.stash.lock:
        requests = request.server.stash.take(REQUESTS)
        if not requests:
            requests = []
        requests.append(obj)
        request.server.stash.put(REQUESTS, requests)
    return None


def get_requests(request) -> str:
    with request.server.stash.lock:
        return json.dumps(request.server.stash.take(REQUESTS))


def main(request, response):
    """
    For most requests, simply returns a 200. Actual source/trigger registration
    headers are piped using the `pipe` query param.

    If a `clear-stash` param is set, it will clear the stash.
    """
    if request.GET.get(b"clear-stash"):
        request.stash.take(REQUESTS)
        return

    # We dont want to redirect preflight requests. The cors headers are piped
    # so we can simply return a 200 and redirect the following request
    if request.method == "OPTIONS":
        response.status = 200
        return

    if request.GET.get(b"get-requests"):
        return get_requests(request)

    if request.GET.get(b"store-request"):
        store_request(request)
        return ""



from tests.support.asserts import assert_success
from tests.support.helpers import is_element_in_viewport


def element_send_keys(session, element, text):
    return session.transport.send(
        "POST", "/session/{session_id}/element/{element_id}/value".format(
            session_id=session.session_id,
            element_id=element.id),
        {"text": text})


def test_element_outside_of_not_scrollable_viewport(session, inline):
    session.url = inline("<input style=\"position: relative; left: -9999px;\">")
    element = session.find.css("input", all=False)

    response = element_send_keys(session, element, "foo")
    assert_success(response)

    assert not is_element_in_viewport(session, element)


def test_element_outside_of_scrollable_viewport(session, inline):
    session.url = inline("<input style=\"margin-top: 102vh;\">")
    element = session.find.css("input", all=False)

    response = element_send_keys(session, element, "foo")
    assert_success(response)

    assert is_element_in_viewport(session, element)


def test_contenteditable_element_outside_of_scrollable_viewport(session, inline):
    session.url = inline("<div contenteditable style=\"margin-top: 102vh;\"></div>")
    element = session.find.css("div", all=False)

    response = element_send_keys(session, element, "foo")
    assert_success(response)

    assert is_element_in_viewport(session, element)



import pytest
from .. import assert_cookie_is_set, create_cookie, get_default_partition_key

pytestmark = pytest.mark.asyncio


@pytest.mark.parametrize(
    "secure",
    [
        True,
        False,
        None
    ]
)
async def test_cookie_secure(bidi_session, set_cookie, test_page, domain_value, secure):
    set_cookie_result = await set_cookie(
        cookie=create_cookie(domain=domain_value(), secure=secure))

    assert set_cookie_result == {
        'partitionKey': (await get_default_partition_key(bidi_session)),
    }

    # `secure` defaults to `false`.
    expected_secure = secure if secure is not None else False
    await assert_cookie_is_set(bidi_session, domain=domain_value(), secure=expected_secure)



"""Methods for the report-event-attribution and report-aggregate-attribution endpoints"""
import json
from typing import List, Optional, Tuple
import urllib.parse

from wptserve.request import Request
from wptserve.stash import Stash
from wptserve.utils import isomorphic_decode, isomorphic_encode

# Key used to access the reports in the stash.
REPORTS = "4691a2d7fca5430fb0f33b1bd8a9d388"
REDIRECT = "9250f93f-2c05-4aae-83b9-2817b0e18b4e"

CLEAR_STASH = isomorphic_encode("clear_stash")
CONFIG_REDIRECT = isomorphic_encode("redirect_to")

Header = Tuple[str, str]
Status = Tuple[int, str]
Response = Tuple[Status, List[Header], str]

def decode_headers(headers: dict) -> dict:
  """Decodes the headers from wptserve.

  wptserve headers are encoded like
  {
    encoded(key): [encoded(value1), encoded(value2),...]
  }
  This method decodes the above using the wptserve.utils.isomorphic_decode
  method
  """
  return {
      isomorphic_decode(key): [isomorphic_decode(el) for el in value
                              ] for key, value in headers.items()
  }

def get_request_origin(request: Request) -> str:
  return "%s://%s" % (request.url_parts.scheme,
                      request.url_parts.netloc)

def configure_redirect(request, origin) -> None:
  with request.server.stash.lock:
      request.server.stash.put(REDIRECT, origin)
      return None

def get_report_redirect_url(request):
  with request.server.stash.lock:
      origin = request.server.stash.take(REDIRECT)
      if origin is None:
         return None
      origin_parts = urllib.parse.urlsplit(origin)
      parts = request.url_parts
      new_parts = origin_parts._replace(path=bytes(parts.path, 'utf-8'))
      return urllib.parse.urlunsplit(new_parts)

def handle_post_report(request: Request, headers: List[Header]) -> Response:
  """Handles POST request for reports.

  Retrieves the report from the request body and stores the report in the
  stash. If clear_stash is specified in the query params, clears the stash.
  """
  if request.GET.get(CLEAR_STASH):
    clear_stash(request.server.stash)
    return (200, "OK"), headers, json.dumps({
        "code": 200,
        "message": "Stash successfully cleared.",
    })

  redirect_origin = request.GET.get(CONFIG_REDIRECT)
  if redirect_origin:
    configure_redirect(request, redirect_origin)
    return (200, "OK"), headers, json.dumps({
        "code": 200,
        "message": "Redirect successfully configured.",
    })

  redirect_url = get_report_redirect_url(request)
  if redirect_url is not None:
    headers.append(("Location", redirect_url))
    return (308, "Permanent Redirect"), headers, json.dumps({
        "code": 308
    })

  store_report(
      request.server.stash, get_request_origin(request), {
          "body": request.body.decode("utf-8"),
          "headers": decode_headers(request.headers)
      })
  return (201, "OK"), headers, json.dumps({
      "code": 201,
      "message": "Report successfully stored."
  })


def handle_get_reports(request: Request, headers: List[Header]) -> Response:
  """Handles GET request for reports.

  Retrieves and returns all reports from the stash.
  """
  reports = take_reports(request.server.stash, get_request_origin(request))
  headers.append(("Access-Control-Allow-Origin", "*"))
  return (200, "OK"), headers, json.dumps({
      "code": 200,
      "reports": reports,
  })


def store_report(stash: Stash, origin: str, report: str) -> None:
  """Stores the report in the stash. Report here is a JSON."""
  with stash.lock:
    reports_dict = stash.take(REPORTS)
    if not reports_dict:
      reports_dict = {}
    reports = reports_dict.get(origin, [])
    reports.append(report)
    reports_dict[origin] = reports
    stash.put(REPORTS, reports_dict)
  return None

def clear_stash(stash: Stash) -> None:
  "Clears the stash."
  stash.take(REPORTS)
  stash.take(REDIRECT)
  return None

def take_reports(stash: Stash, origin: str) -> List[str]:
  """Takes all the reports from the stash and returns them."""
  with stash.lock:
    reports_dict = stash.take(REPORTS)
    if not reports_dict:
      reports_dict = {}

    reports = reports_dict.pop(origin, [])
    stash.put(REPORTS, reports_dict)
  return reports


def handle_reports(request: Request) -> Response:
  """Handles request to get or store reports."""
  headers = [("Content-Type", "application/json")]
  if request.method == "POST":
    return handle_post_report(request, headers)
  if request.method == "GET":
    return handle_get_reports(request, headers)
  return (405, "Method Not Allowed"), headers, json.dumps({
      "code": 405,
      "message": "Only GET or POST methods are supported."
  })



import sys

platform_name = {
    # From Python version 3.3: On Linux, sys.platform doesn't contain the major version anymore.
    # It is always 'linux'. See
    # https://docs.python.org/3/library/sys.html#sys.platform
    "linux": "linux",
    "linux2": "linux",
    "win32": "windows",
    "cygwin": "windows",
    "darwin": "mac"
}.get(sys.platform)



def main(request, response):
    if request.headers.get(b"Origin") is not None:
        response.headers.set(
            b"Access-Control-Allow-Origin", request.headers.get(b"Origin")
        )
    return ""



import pytest

from tests.support import platform_name
from tests.support.asserts import assert_success


@pytest.mark.skipif(platform_name is None, reason="Unsupported platform {}".format(platform_name))
def test_corresponds_to_local_system(new_session, add_browser_capabilities):
    response, _ = new_session({"capabilities": {"alwaysMatch": add_browser_capabilities({})}})
    value = assert_success(response)
    assert value["capabilities"]["platformName"] == platform_name



import pytest
from .. import assert_cookie_is_not_set, assert_cookie_is_set, create_cookie, get_default_partition_key
from datetime import datetime, timedelta
import time

pytestmark = pytest.mark.asyncio


async def test_cookie_expiry_unset(bidi_session, set_cookie, test_page, domain_value):
    set_cookie_result = await set_cookie(
        cookie=create_cookie(
            domain=domain_value(),
            expiry=None))

    assert set_cookie_result == {
        'partitionKey': (await get_default_partition_key(bidi_session)),
    }

    await assert_cookie_is_set(bidi_session, expiry=None, domain=domain_value())


async def test_cookie_expiry_future(bidi_session, set_cookie, test_page, domain_value):
    tomorrow = datetime.now() + timedelta(1)
    tomorrow_timestamp = time.mktime(tomorrow.timetuple())

    set_cookie_result = await set_cookie(
        cookie=create_cookie(
            domain=domain_value(),
            expiry=tomorrow_timestamp))

    assert set_cookie_result == {
        'partitionKey': (await get_default_partition_key(bidi_session)),
    }

    await assert_cookie_is_set(bidi_session, expiry=tomorrow_timestamp, domain=domain_value())


async def test_cookie_expiry_past(bidi_session, set_cookie, test_page, domain_value):
    yesterday = datetime.now() - timedelta(1)
    yesterday_timestamp = time.mktime(yesterday.timetuple())

    set_cookie_result = await set_cookie(
        cookie=create_cookie(
            domain=domain_value(),
            expiry=yesterday_timestamp))

    assert set_cookie_result == {
        'partitionKey': (await get_default_partition_key(bidi_session)),
    }

    await assert_cookie_is_not_set(bidi_session)



import pytest

import asyncio

from .. import assert_response_event, AUTH_REQUIRED_EVENT, PAGE_EMPTY_HTML


@pytest.mark.asyncio
async def test_subscribe_status(
    bidi_session, new_tab, subscribe_events, wait_for_event, wait_for_future_safe, url, fetch
):
    await bidi_session.browsing_context.navigate(
        context=new_tab["context"],
        url=url(PAGE_EMPTY_HTML),
        wait="complete",
    )

    await subscribe_events(events=[AUTH_REQUIRED_EVENT])

    # Track all received network.authRequired events in the events array.
    events = []

    async def on_event(method, data):
        events.append(data)

    remove_listener = bidi_session.add_event_listener(
        AUTH_REQUIRED_EVENT, on_event)

    auth_url = url(
        "/webdriver/tests/support/http_handlers/authentication.py?realm=testrealm"
    )

    on_auth_required = wait_for_event(AUTH_REQUIRED_EVENT)

    asyncio.ensure_future(fetch(url=auth_url, context=new_tab))

    await wait_for_future_safe(on_auth_required)

    assert len(events) == 1
    expected_request = {"method": "GET", "url": auth_url}
    expected_response = {
        "url": auth_url,
        "authChallenges": [
            ({"scheme": "Basic", "realm": "testrealm"}),
        ],
    }
    assert_response_event(
        events[0],
        expected_request=expected_request,
        expected_response=expected_response,
        redirect_count=0,
    )

    remove_listener()


@pytest.mark.asyncio
async def test_no_authentication(
    bidi_session, new_tab, subscribe_events, url
):
    await subscribe_events(events=[AUTH_REQUIRED_EVENT])

    # Track all received network.authRequired events in the events array.
    events = []

    async def on_event(method, data):
        events.append(data)

    remove_listener = bidi_session.add_event_listener(
        AUTH_REQUIRED_EVENT, on_event)

    # Navigate to a page which should not trigger any authentication.
    await bidi_session.browsing_context.navigate(
        context=new_tab["context"],
        url=url(PAGE_EMPTY_HTML),
        wait="complete",
    )

    assert len(events) == 0
    remove_listener()



from urllib.parse import urlunsplit

import pytest


@pytest.fixture
def origin(server_config, domain_value):
    def origin(protocol="https", domain="", subdomain=""):
        return urlunsplit((protocol, domain_value(domain, subdomain), "", "", ""))

    return origin



import json

from webdriver.bidi.modules.script import ContextTarget

async def get_object_from_context(bidi_session, context, object_path):
    """Return a plain JS object from a given context, accessible at the given object_path"""
    events_str = await bidi_session.script.evaluate(
        expression=f"JSON.stringify({object_path})",
        target=ContextTarget(context),
        await_promise=False,
    )
    return json.loads(events_str["value"])


async def get_events(bidi_session, context):
    """Return list of key events recorded on the test_actions.html page."""
    events = await get_object_from_context(bidi_session, context, "allEvents.events")

    # `key` values in `allEvents` may be escaped (see `escapeSurrogateHalf` in
    # test_actions.html), so this converts them back into unicode literals.
    for e in events:
        # example: turn "U+d83d" (6 chars) into u"\ud83d" (1 char)
        if "key" in e and e["key"].startswith("U+"):
            key = e["key"]
            hex_suffix = key[key.index("+") + 1:]
            e["key"] = chr(int(hex_suffix, 16))

        # WebKit sets code as 'Unidentified' for unidentified key codes, but
        # tests expect ''.
        if "code" in e and e["code"] == "Unidentified":
            e["code"] = ""
    return events


async def get_keys_value(bidi_session, context):
    keys_value = await bidi_session.script.evaluate(
        expression="""document.getElementById("keys").value""",
        target=ContextTarget(context),
        await_promise=False,
    )

    return keys_value["value"]



import pytest
from webdriver.bidi.error import InvalidSessionIDError

pytestmark = pytest.mark.asyncio


async def test_without_session(bidi_client):
    # Connect the client.
    bidi_session = await bidi_client()
    response = await bidi_session.session.status()

    assert response["ready"] is True

    with pytest.raises(InvalidSessionIDError):
        await bidi_session.session.end()



# This will echo the 'Sec-Required-Document-Policy' request header in the body
# of the response, as well as in the 'Document-Policy' response header (to
# ensure the response is loaded by a user agent which is implementing document
# policy.)
import json

from wptserve.utils import isomorphic_decode

def main(request, response):
    msg = {}
    headers = [(b'Content-Type', b'text/html')]

    srdp = request.headers.get(b'Sec-Required-Document-Policy')
    if srdp:
      msg[u'requiredPolicy'] = isomorphic_decode(srdp)
      headers.append((b'Document-Policy', srdp))

    frameId = request.GET.first(b'id',None)
    if frameId:
      msg[u'id'] = isomorphic_decode(frameId)

    content = u"""<!DOCTYPE html>
<script>
top.postMessage(%s, "*");
</script>
%s
""" % (json.dumps(msg), isomorphic_decode(srdp) if srdp != None else srdp)

    return (200, u'OK'), headers, content




import pytest

from . import navigate_and_assert

pytestmark = pytest.mark.asyncio


@pytest.mark.parametrize(
    "url",
    [
        "thisprotocoldoesnotexist://",
        "https://doesnotexist.localhost/",
        "https://localhost:0",
    ],
    ids=[
        "protocol",
        "host",
        "port",
    ]
)
async def test_invalid_address(bidi_session, new_tab, url):
    await navigate_and_assert(bidi_session, new_tab, url, expected_error=True)



def main(request, response):
    headers = []
    if b"headers" in request.GET:
        checked_headers = request.GET.first(b"headers").split(b"|")
        for header in checked_headers:
            if header in request.headers:
                headers.append((b"x-request-" + header, request.headers.get(header, b"")))

    if b"cors" in request.GET:
        if b"Origin" in request.headers:
            headers.append((b"Access-Control-Allow-Origin", request.headers.get(b"Origin", b"")))
        else:
            headers.append((b"Access-Control-Allow-Origin", b"*"))
        headers.append((b"Access-Control-Allow-Credentials", b"true"))
        headers.append((b"Access-Control-Allow-Methods", b"GET, POST, HEAD"))
        exposed_headers = [b"x-request-" + header for header in checked_headers]
        headers.append((b"Access-Control-Expose-Headers", b", ".join(exposed_headers)))
        if b"allow_headers" in request.GET:
            headers.append((b"Access-Control-Allow-Headers", request.GET[b'allow_headers']))
        else:
            headers.append((b"Access-Control-Allow-Headers", b", ".join(request.headers)))

    headers.append((b"content-type", b"text/plain"))
    return headers, b""



import json

from wptserve.utils import isomorphic_decode

def main(request, response):
    key = request.GET.first(b"stash")
    origin = request.headers.get(b"origin")
    if origin is None:
        origin = b"no Origin header"

    origin_list = request.server.stash.take(key)

    if b"dump" in request.GET:
        response.headers.set(b"Content-Type", b"application/json")
        response.content = json.dumps(origin_list)
        return

    if origin_list is None:
        origin_list = [isomorphic_decode(origin)]
    else:
        origin_list.append(isomorphic_decode(origin))

    request.server.stash.put(key, origin_list)

    if b"location" in request.GET:
        location = request.GET.first(b"location")
        if b"dummyJS" in request.GET:
            location += b"&dummyJS"
        response.status = 308
        response.headers.set(b"Location", location)
        return

    response.headers.set(b"Content-Type", b"text/html")
    response.headers.set(b"Access-Control-Allow-Origin", b"*")
    if b"dummyJS" in request.GET:
        response.content = b"console.log('dummy JS')"
    else:
        response.content = b"<meta charset=utf-8>\n<body><script>parent.postMessage('loaded','*')</script></body>"



"""An HTTP request handler for WPT that handles /set_beacon.py requests."""

_BEACON_ID_KEY = b"uuid"
_BEACON_DATA_PATH = "beacon_data"
_BEACON_FORM_PAYLOAD_KEY = b"payload"
_BEACON_BODY_PAYLOAD_KEY = "payload="
_BEACON_EXPECT_ORIGIN_KEY = b"expectOrigin"
_BEACON_EXPECT_PREFLIGHT_KEY = b"expectPreflight"
_BEACON_EXPECT_CREDS_KEY = b"expectCredentials"


def main(request, response):
    """Stores the given beacon's data keyed by uuid in the server.

    For GET request, this handler assumes no data.
    For POST request, this handler extracts data from request body:
      - Content-Type=multipart/form-data: data keyed by 'payload'.
      - the entire request body.

    Multiple data can be added for the same uuid.

    The data is stored as UTF-8 format.
    """
    if _BEACON_ID_KEY not in request.GET:
        response.status = 400
        return "Must provide a UUID to store beacon data"
    uuid = request.GET.first(_BEACON_ID_KEY)

    expected_origin = request.GET.get(_BEACON_EXPECT_ORIGIN_KEY)
    if b"origin" in request.headers:
        origin = request.headers.get(b"origin")
        if expected_origin:
            assert origin == expected_origin, f"expected {expected_origin}, got {origin}"
        response.headers.set(b"Access-Control-Allow-Origin", origin)
    else:
        assert expected_origin is None, f"expected None, got {expected_origin}"

    # Handles preflight request first.
    if request.method == u"OPTIONS":
        assert request.GET.get(
            _BEACON_EXPECT_PREFLIGHT_KEY) == b"true", "Preflight not expected."

        # preflight must not have cookies.
        assert b"Cookie" not in request.headers

        requested_headers = request.headers.get(
            b"Access-Control-Request-Headers")
        assert b"content-type" in requested_headers, f"expected content-type, got {requested_headers}"
        response.headers.set(b"Access-Control-Allow-Headers", b"content-type")

        requested_method = request.headers.get(b"Access-Control-Request-Method")
        assert requested_method == b"POST", f"expected POST, got {requested_method}"
        response.headers.set(b"Access-Control-Allow-Methods", b"POST")

        return response

    expect_creds = request.GET.get(_BEACON_EXPECT_CREDS_KEY) == b"true"
    if expect_creds:
        assert b"Cookie" in request.headers
    else:
        assert b"Cookie" not in request.headers

    data = None
    if request.method == u"POST":
        if b"multipart/form-data" in request.headers.get(b"Content-Type", b""):
            if _BEACON_FORM_PAYLOAD_KEY in request.POST:
                data = request.POST.first(_BEACON_FORM_PAYLOAD_KEY).decode(
                    'utf-8')
        elif request.body:
            data = request.body.decode('utf-8')
            if data.startswith(_BEACON_BODY_PAYLOAD_KEY):
                data = data.split(_BEACON_BODY_PAYLOAD_KEY)[1]

    with request.server.stash.lock:
        saved_data = request.server.stash.take(key=uuid, path=_BEACON_DATA_PATH)
        if not saved_data:
            saved_data = [data]
        else:
            saved_data.append(data)
        request.server.stash.put(
            key=uuid, value=saved_data, path=_BEACON_DATA_PATH)

    response.status = 200



import json

def main(request, response):
    response.headers.set(b"Access-Control-Allow-Origin", b"*")
    match = b"/fetch/compression-dictionary/resources/*"
    content = b"This is a test dictionary.\n"
    if b"match" in request.GET:
        match = request.GET.first(b"match")
    if b"content" in request.GET:
        content = request.GET.first(b"content")

    token = request.GET.first(b"save_header", None)
    if token is not None:
        headers = {}
        for header in request.headers:
            key = header.decode('utf-8')
            value = request.headers.get(header).decode('utf-8')
            headers[key] = value
        with request.server.stash.lock:
            request.server.stash.put(token, json.dumps(headers))

    previous_token = request.GET.first(b"get_previous_header", None)
    if previous_token is not None:
        result = {}
        with request.server.stash.lock:
            store = request.server.stash.take(previous_token)
            if store is not None:
                headers = json.loads(store)
                result["headers"] = headers
            return json.dumps(result)

    options = b"match=\"" + match + b"\""
    if b"id" in request.GET:
        options += b", id=\"" + request.GET.first(b"id") + b"\""
    response.headers.set(b"Use-As-Dictionary", options)
    response.headers.set(b"Cache-Control", b"max-age=3600")
    return content



import pytest

pytestmark = pytest.mark.asyncio


async def test_close_browser(new_session, add_browser_capabilities):
    bidi_session = await new_session(
        capabilities={"alwaysMatch": add_browser_capabilities({})}
    )

    await bidi_session.browser.close()

    # Wait for the browser to actually close.
    bidi_session.current_browser.wait()

    assert bidi_session.current_browser.is_running is False


async def test_start_session_again(new_session, add_browser_capabilities):
    bidi_session = await new_session(
        capabilities={"alwaysMatch": add_browser_capabilities({})}
    )
    first_session_id = bidi_session.session_id

    await bidi_session.browser.close()

    # Wait for the browser to actually close.
    bidi_session.current_browser.wait()

    # Try to create a session again.
    bidi_session = await new_session(
        capabilities={"alwaysMatch": add_browser_capabilities({})}
    )

    assert isinstance(bidi_session.session_id, str)
    assert first_session_id != bidi_session.session_id



import pytest

from webdriver.bidi.modules.input import Actions, get_element_origin

from .. import get_events
from . import (
    assert_pointer_events,
    get_inview_center_bidi,
    get_shadow_root_from_test_page,
    record_pointer_events,
)

pytestmark = pytest.mark.asyncio


@pytest.mark.parametrize("mode", ["open", "closed"])
@pytest.mark.parametrize("nested", [False, True], ids=["outer", "inner"])
async def test_touch_pointer_in_shadow_tree(
    bidi_session, top_context, get_test_page, mode, nested
):
    await bidi_session.browsing_context.navigate(
        context=top_context["context"],
        url=get_test_page(
            shadow_doc="""
            <div id="pointer-target"
                 style="width: 10px; height: 10px; background-color:blue;">
            </div>""",
            shadow_root_mode=mode,
            nested_shadow_dom=nested,
        ),
        wait="complete",
    )

    shadow_root = await get_shadow_root_from_test_page(
        bidi_session, top_context, nested
    )

    # Add a simplified event recorder to track events in the test ShadowRoot.
    target = await record_pointer_events(
        bidi_session, top_context, shadow_root, "#pointer-target"
    )

    actions = Actions()
    (
        actions.add_pointer(pointer_type="touch")
        .pointer_move(x=0, y=0, origin=get_element_origin(target))
        .pointer_down(button=0)
        .pointer_up(button=0)
    )

    await bidi_session.input.perform_actions(
        actions=actions, context=top_context["context"]
    )

    await assert_pointer_events(
        bidi_session,
        top_context,
        expected_events=["pointerdown", "pointerup"],
        target="pointer-target",
        pointer_type="touch",
    )


async def test_touch_pointer_properties(
    bidi_session, top_context, get_element, load_static_test_page
):
    await load_static_test_page(page="test_actions_pointer.html")

    pointerArea = await get_element("#pointerArea")
    center = await get_inview_center_bidi(
        bidi_session, context=top_context, element=pointerArea
    )

    actions = Actions()
    (
        actions.add_pointer(pointer_type="touch")
        .pointer_move(x=0, y=0, origin=get_element_origin(pointerArea))
        .pointer_down(
            button=0,
            width=23,
            height=31,
            pressure=0.78,
            twist=355,
        )
        .pointer_move(
            x=10,
            y=10,
            origin=get_element_origin(pointerArea),
            width=39,
            height=35,
            pressure=0.91,
            twist=345,
        )
        .pointer_up(button=0)
        .pointer_move(x=80, y=50, origin=get_element_origin(pointerArea))
    )

    await bidi_session.input.perform_actions(
        actions=actions, context=top_context["context"]
    )

    events = await get_events(bidi_session, top_context["context"])

    assert len(events) == 7
    event_types = [e["type"] for e in events]
    assert [
        "pointerover",
        "pointerenter",
        "pointerdown",
        "pointermove",
        "pointerup",
        "pointerout",
        "pointerleave",
    ] == event_types
    assert events[2]["type"] == "pointerdown"
    assert events[2]["pageX"] == pytest.approx(center["x"], abs=1.0)
    assert events[2]["pageY"] == pytest.approx(center["y"], abs=1.0)
    assert events[2]["target"] == "pointerArea"
    assert events[2]["pointerType"] == "touch"
    assert round(events[2]["width"], 2) == 23
    assert round(events[2]["height"], 2) == 31
    assert round(events[2]["pressure"], 2) == 0.78
    assert events[3]["type"] == "pointermove"
    assert events[3]["pageX"] == pytest.approx(center["x"] + 10, abs=1.0)
    assert events[3]["pageY"] == pytest.approx(center["y"] + 10, abs=1.0)
    assert events[3]["target"] == "pointerArea"
    assert events[3]["pointerType"] == "touch"
    assert round(events[3]["width"], 2) == 39
    assert round(events[3]["height"], 2) == 35
    assert round(events[3]["pressure"], 2) == 0.91


async def test_touch_pointer_properties_angle_twist(
    bidi_session, top_context, get_element, load_static_test_page
):
    await load_static_test_page(page="test_actions_pointer.html")

    pointerArea = await get_element("#pointerArea")
    await get_inview_center_bidi(
        bidi_session, context=top_context, element=pointerArea
    )

    actions = Actions()
    (
        actions.add_pointer(pointer_type="touch")
        .pointer_move(x=0, y=0, origin=get_element_origin(pointerArea))
        .pointer_down(
            button=0,
            width=23,
            height=31,
            pressure=0.78,
            altitude_angle=1.2,
            azimuth_angle=6,
            twist=355,
        )
        .pointer_move(
            x=10,
            y=10,
            origin=get_element_origin(pointerArea),
            width=39,
            height=35,
            pressure=0.91,
            altitude_angle=0.5,
            azimuth_angle=1.8,
            twist=345,
        )
        .pointer_up(button=0)
        .pointer_move(x=80, y=50, origin=get_element_origin(pointerArea))
    )

    await bidi_session.input.perform_actions(
        actions=actions, context=top_context["context"]
    )

    events = await get_events(bidi_session, top_context["context"])

    assert len(events) == 7
    event_types = [e["type"] for e in events]
    assert [
        "pointerover",
        "pointerenter",
        "pointerdown",
        "pointermove",
        "pointerup",
        "pointerout",
        "pointerleave",
    ] == event_types
    assert events[2]["type"] == "pointerdown"
    assert events[2]["tiltX"] == 20
    assert events[2]["tiltY"] == -6
    assert events[2]["twist"] == 355
    assert events[3]["type"] == "pointermove"
    assert events[3]["tiltX"] == -23
    assert events[3]["tiltY"] == 61
    assert events[3]["twist"] == 345



import pytest

from webdriver.error import NoSuchWindowException

import time
from tests.classic.perform_actions.support.refine import get_events
from tests.support.keys import Keys
from tests.support.sync import Poll


def test_null_response_value(session, wheel_chain):
    value = wheel_chain.scroll(0, 0, 0, 10).perform()
    assert value is None


def test_no_top_browsing_context(session, closed_window, wheel_chain):
    with pytest.raises(NoSuchWindowException):
        wheel_chain.scroll(0, 0, 0, 10).perform()


def test_no_browsing_context(session, closed_window, wheel_chain):
    with pytest.raises(NoSuchWindowException):
        wheel_chain.scroll(0, 0, 0, 10).perform()


def test_scroll_not_scrollable(session, test_actions_scroll_page, wheel_chain):
    target = session.find.css("#not-scrollable", all=False)

    wheel_chain.scroll(0, 0, 5, 10, origin=target).perform()

    events = get_events(session)
    assert len(events) == 1
    assert events[0]["type"] == "wheel"
    assert events[0]["deltaX"] == 5
    assert events[0]["deltaY"] == 10
    assert events[0]["deltaZ"] == 0
    assert events[0]["target"] == "not-scrollable-content"


def test_scroll_scrollable_overflow(session, test_actions_scroll_page, wheel_chain):
    target = session.find.css("#scrollable", all=False)

    wheel_chain.scroll(0, 0, 5, 10, origin=target).perform()

    events = get_events(session)
    assert len(events) == 1
    assert events[0]["type"] == "wheel"
    assert events[0]["deltaX"] == 5
    assert events[0]["deltaY"] == 10
    assert events[0]["deltaZ"] == 0
    assert events[0]["target"] == "scrollable-content"


def test_scroll_iframe(session, test_actions_scroll_page, wheel_chain):
    target = session.find.css("#iframe", all=False)

    wheel_chain.scroll(0, 0, 5, 10, origin=target).perform()

    # Chrome requires some time (~10-20ms) to process the event from the iframe, so we wait for it.
    def wait_for_events(_):
        return len(get_events(session)) > 0

    Poll(session, timeout=0.5, interval=0.01, message='No wheel events found').until(wait_for_events)
    events = get_events(session)

    assert len(events) == 1
    assert events[0]["type"] == "wheel"
    assert events[0]["deltaX"] == 5
    assert events[0]["deltaY"] == 10
    assert events[0]["deltaZ"] == 0
    assert events[0]["target"] == "iframeContent"


@pytest.mark.parametrize("mode", ["open", "closed"])
@pytest.mark.parametrize("nested", [False, True], ids=["outer", "inner"])
def test_scroll_shadow_tree(session, get_test_page, wheel_chain, mode, nested):
    session.url = get_test_page(
        shadow_doc="""
        <div id="scrollableShadowTree"
             style="width: 100px; height: 100px; overflow: auto;">
            <div
                id="scrollableShadowTreeContent"
                style="width: 600px; height: 1000px; background-color:blue"></div>
        </div>""",
        shadow_root_mode=mode,
        nested_shadow_dom=nested,
    )

    shadow_root = session.find.css("custom-element", all=False).shadow_root

    if nested:
        shadow_root = shadow_root.find_element(
            "css selector", "inner-custom-element"
        ).shadow_root

    scrollable = shadow_root.find_element("css selector", "#scrollableShadowTree")

    # Add a simplified event recorder to track events in the test ShadowRoot.
    session.execute_script(
        """
        window.wheelEvents = [];
        arguments[0].addEventListener("wheel",
            function(event) {
                window.wheelEvents.push({
                    "deltaX": event.deltaX,
                    "deltaY": event.deltaY,
                    "target": event.target.id
                });
            }
        );
    """,
        args=(scrollable,),
    )

    wheel_chain.scroll(0, 0, 5, 10, origin=scrollable).perform()

    events = session.execute_script("return window.wheelEvents;") or []
    assert len(events) == 1
    assert events[0]["deltaX"] == 5
    assert events[0]["deltaY"] == 10
    assert events[0]["target"] == "scrollableShadowTreeContent"


def test_scroll_with_key_pressed(
    session, test_actions_scroll_page, key_chain, wheel_chain
):
    scrollable = session.find.css("#scrollable", all=False)

    key_chain.key_down(Keys.R_SHIFT).perform()
    wheel_chain.scroll(0, 0, 5, 10, origin=scrollable).perform()
    key_chain.key_up(Keys.R_SHIFT).perform()

    events = get_events(session)
    assert len(events) == 1
    assert events[0]["type"] == "wheel"
    assert events[0]["shiftKey"] == True



def main(request, response):
    user = request.auth.username
    password = request.auth.password

    if user == b"user" and password == b"password":
        return b"Authentication done"

    realm = b"test"
    if b"realm" in request.GET:
        realm = request.GET.first(b"realm")

    return ((401, b"Unauthorized"),
            [(b"WWW-Authenticate", b'Basic realm="' + realm + b'"')],
            b"Please login with credentials 'user' and 'password'")
